@@ -81,11 +81,15 @@ ChannelDesc::ChannelDesc( const ChannelDesc &chan )
 	maxapid = chan.maxapid;
 	for ( int i=0; i<maxapid; i++ ) {
 		apid[i] = chan.apid[i];
+		apid[i].ac3 = chan.apid[i].ac3;
+		apid[i].lang = chan.apid[i].lang;
 	}
 	napid = chan.napid;
 	maxsubpid = chan.maxsubpid;
 	for ( int i=0; i<maxsubpid; i++ ) {
 		subpid[i] = chan.subpid[i];
+		subpid[i].type = chan.subpid[i].type;
+		subpid[i].lang = chan.subpid[i].lang;
 	}
 	nsubpid = chan.nsubpid;
 }
@@ -101,7 +105,7 @@ Transponder::Transponder()
 	freq = 0;
 	sr = 0;
 	pol = 'v';
-	tsid = 0;
+	nid=tsid = 0;
 	inversion=INVERSION_AUTO;
 	modulation=QAM_AUTO;
 	hierarchy=HIERARCHY_AUTO;
@@ -111,6 +115,8 @@ Transponder::Transponder()
 	coderateH=FEC_AUTO;
 	bandwidth=BANDWIDTH_AUTO;
 	snr = 0;
+	rolloff = ROLLOFF_AUTO;
+	S2 = 0;
 }
 
 Transponder::Transponder( const Transponder &trans )
@@ -121,6 +127,7 @@ Transponder::Transponder( const Transponder &trans )
 	sr = trans.sr;
 	pol = trans.pol;
 	tsid = trans.tsid;
+	nid = trans.nid;
 	inversion=trans.inversion;
 	modulation=trans.modulation;
 	hierarchy=trans.hierarchy;
@@ -129,6 +136,8 @@ Transponder::Transponder( const Transponder &trans )
 	coderateL=trans.coderateL;
 	coderateH=trans.coderateH;
 	bandwidth=trans.bandwidth;
+	rolloff = trans.rolloff;
+	S2 = trans.S2;
 }
 
 bool Transponder::sameAs( Transponder *trans )
@@ -177,57 +186,3 @@ bool Transponder::operator!=( const Transponder &t )
 Transponder::~Transponder()
 {
 }
-
-ShortEvent::ShortEvent()
-{
-	name=text="";
-}
-
-ShortEvent::~ShortEvent()
-{
-}
-
-EventDesc::EventDesc()
-{
-	loop = 0;
-	sid=tsid=eid=0;
-	tid=sn=lsn=running=0;
-	title=subtitle="";
-	source="";
-	shortEvents.setAutoDelete( true );
-	extEvents.setAutoDelete( true );
-}
-
-void EventDesc::printOut()
-{
-	/*int i;
-	ShortEvent *ev;
-
-	fprintf(stderr,"Program : %d\n", sid);
-	fprintf(stderr,"%s\n", startDateTime.latin1() );
-	fprintf(stderr,"%s\n", duration.latin1() );
-	switch (running) {
-		case 4 :
-			fprintf(stderr,"Running\n");
-			break;
-		default :
-			fprintf(stderr,"Not running\n");
-			break;
-	}
-	for ( i=0; i<shortEvents.count(); i++ ) {
-		ev = shortEvents.at(i);
-		fprintf(stderr,"%s\n", ev->name.latin1() );
-		fprintf(stderr,"%s\n", ev->text.latin1() );
-	}
-	for ( i=0; i<extEvents.count(); i++ ) {
-		fprintf(stderr,"%s", extEvents.at(i)->latin1() );
-		//fprintf(stderr,"\n" );
-	}
-	fprintf(stderr,"\n\n" );*/
-}
-
-EventDesc::~EventDesc()
-{
-	shortEvents.clear();
-	extEvents.clear();
-}
