@@ -1,7 +1,7 @@
 /*
  * dvbconfig.cpp
  *
- * Copyright (C) 2004-2006 Christophe Thommeret <hftom@free.fr>
+ * Copyright (C) 2004-2007 Christophe Thommeret <hftom@free.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,8 +23,6 @@
 #include <unistd.h>
 #include <errno.h>
 
-#include <linux/dvb/frontend.h>
-
 #include <qdir.h>
 #include <qlayout.h>
 #include <qgroupbox.h>
@@ -39,8 +37,8 @@
 #include <kstandarddirs.h>
 
 #include "dvbconfig.h"
-#include "gdvb.h"
 #include "kaffeinedvbplugin.h"
+#include "dvbpanel.h"
 
 
 
@@ -75,6 +73,21 @@ void MPushButton::isClicked()
 
 
 
+MCAMButton::MCAMButton( QWidget *parent, int devNum ) : QPushButton( i18n("CAM"), parent )
+{
+	deviceNumber = devNum;
+	connect( this, SIGNAL(clicked()), this, SLOT(isClicked()) );
+}
+
+
+
+void MCAMButton::isClicked()
+{
+	emit clicked( deviceNumber );
+}
+
+
+
 MComboBox::MComboBox( QWidget *parent, int devNum, int lnbNum ) : QComboBox( parent )
 {
 	deviceNumber = devNum;
@@ -112,6 +125,12 @@ Device::Device( int anum, int tnum, fe_type_t t, const QString &n, bool as )
 	source = "";
 	canAutoscan= as;
 	tuningTimeout = 1500;
+	hasCAM = false;
+	camMaxService = 1;
+	secMini = 0;
+	secTwice = 0;
+	priority = 10;
+	doS2 = 0;
 }
 
 
@@ -134,8 +153,10 @@ DVBconfig::DVBconfig( const QString &dvbConf )
 		dir.mkdir( dvbConfigIconsDir );
 	config = new KConfig( dvbConfigDir+"dvbrc" );
 	downProgress = 0;
+	sizeFile = 0;
 	categories.setAutoDelete( true );
 	devList.setAutoDelete( true );
+	readFirst();
 	startup();
 	readConfig();
 }
@@ -154,34 +175,27 @@ DVBconfig::~DVBconfig()
 
 bool DVBconfig::haveDvbDevice()
 {
-	int i=0, j=0, res, fdFrontend=0;
+	int i, j, res, fdFrontend=0;
 	struct dvb_frontend_info info;
-	fprintf(stderr,"%d\n", errno);
 
-	QStringList list;
+	QStringList list, flist;
 	QDir d;
 	d.setPath( "/dev/dvb/" );
 	list = d.entryList( "adapter*", QDir::Dirs, QDir::Name );
 
 	for ( i=0; i<(int)list.count(); i++ ) {
-		j = 0;
-		while ( j>=0 && j<4 ) {
-			fdFrontend = open( QString("/dev/dvb/%1/frontend%2").arg( list[i] ).arg( j ).ascii(), O_RDWR);
-			if ( fdFrontend<0 ) {
-				if ( errno==ENOENT )
-					j=-1;
-				else
-					++j;
-			}
-			else {
+		d.setPath( "/dev/dvb/"+list[i]+"/" );
+		flist = d.entryList( "frontend*", QDir::System, QDir::Name );
+		for ( j=0; j<(int)flist.count(); j++ ) {
+			fdFrontend = open( QString("/dev/dvb/%1/%2").arg( list[i] ).arg( flist[j] ).ascii(), O_RDWR);
+			if ( fdFrontend>0 ) {
 				if ( (res = ioctl( fdFrontend, FE_GET_INFO, &info ) < 0) )
-					perror( QString("/dev/dvb/%1/frontend%2 FE_GET_INFO: ").arg( list[i] ).arg( j ).ascii() );
+					perror( QString("/dev/dvb/%1/%2 FE_GET_INFO: ").arg( list[i] ).arg( flist[j] ).ascii() );
 				else {
 					close( fdFrontend );
 					return true;
 				}
 				close( fdFrontend );
-				++j;
 			}
 		}
 	}
@@ -196,47 +210,49 @@ void DVBconfig::startup()
 	int i=0, j=0, res, fdFrontend=0;
 	struct dvb_frontend_info info;
 	bool as;
+	QTime t1;
 
-	QStringList list;
+	QStringList list, flist;
+	QString s, t;
 	QDir d;
 	d.setPath( "/dev/dvb/" );
 	list = d.entryList( "adapter*", QDir::Dirs, QDir::Name );
 
 	for ( i=0; i<(int)list.count(); i++ ) {
-		j = 0;
-		while ( j>=0 && j<4 ) {
-			fdFrontend = open( QString("/dev/dvb/%1/frontend%2").arg( list[i] ).arg( j ).ascii(), O_RDWR);
-			if ( fdFrontend<0 ) {
-				perror( QString("/dev/dvb/%1/frontend%2 : ").arg( list[i] ).arg( j ).ascii() );
-				if ( errno==ENOENT )
-					j=-1;
-				else
-					++j;
-			}
-			else {
+		d.setPath( "/dev/dvb/"+list[i]+"/" );
+		flist = d.entryList( "frontend*", QDir::System, QDir::Name );
+		for ( j=0; j<(int)flist.count(); j++ ) {
+			s = list[i];
+			t = flist[j];
+			if ( devList.count()==MAX_DEVICES )
+				break;
+			if ( !probeMfe && t!="frontend0" )
+				continue;
+			t1 = QTime::currentTime();
+			fdFrontend = open( QString("/dev/dvb/%1/%2").arg( s ).arg( t ).ascii(), O_RDWR);
+			if ( fdFrontend>0 ) {
 				if ( !(res = ioctl( fdFrontend, FE_GET_INFO, &info ) < 0) ) {
-					if ( (info.caps & FE_CAN_FEC_AUTO)
+					if ( (info.type==FE_OFDM)
 						&& (info.caps & FE_CAN_QAM_AUTO)
 						&& (info.caps & FE_CAN_TRANSMISSION_MODE_AUTO)
 						&& (info.caps & FE_CAN_GUARD_INTERVAL_AUTO)
 						&& (info.caps & FE_CAN_HIERARCHY_AUTO)
-						&& (info.type==FE_OFDM) )
+						&& (info.caps & FE_CAN_FEC_AUTO) )
 						as = true;
 					else
 						as = false;
-					devList.append( new Device( list[i].replace("adapter","").toInt(), j, info.type, info.name, as ) );
-					fprintf(stderr,"/dev/dvb/%s/frontend%d : opened ( %s )\n", list[i].ascii(), j, info.name );
+					fprintf(stderr,"/dev/dvb/%s/%s : opened ( %s ) (%dms)\n", s.ascii(), t.ascii(), info.name, t1.msecsTo(QTime::currentTime()) );
+					devList.append( new Device( s.replace("adapter","").toInt(), t.replace("frontend","").toInt(), info.type, info.name, as ) );
 				}
 				close( fdFrontend );
-				++j;
+			}
+			else {
+				perror( QString("/dev/dvb/%1/%2  %3/%4").arg( s ).arg( t ).arg( errno ).arg( -EBUSY ).ascii() );
 			}
 		}
 	}
 
-	//devList.append( new Device( 0, 1, FE_QPSK, "Dummy S", false ) );
-	//devList.append( new Device( 1, 1, FE_OFDM, "Dummy T", false ) );
-	//devList.append( new Device( 2, 0, FE_QAM, "Dummy C", false ) );
-	//devList.append( new Device( 3, 0, FE_ATSC, "Dummy A", false ) );
+	//devList.append( new Device( 3, 0, FE_QPSK, "Dummy S", false ) );
 }
 
 
@@ -306,9 +322,9 @@ bool DVBconfig::localData()
 
 bool DVBconfig::haveData()
 {
-	if ( !QDir( dvbConfigDir+"dvb-s" ).exists() || !QDir( dvbConfigDir+"dvb-c" ).exists() || !QDir( dvbConfigDir+"dvb-t" ).exists() ) {
+	if ( !QDir( dvbConfigDir+"dvb-s" ).exists() || !QDir( dvbConfigDir+"dvb-c" ).exists() || !QDir( dvbConfigDir+"dvb-t" ).exists() || !QDir( dvbConfigDir+"atsc" ).exists()) {
 		loadDvbData(0);
-		if ( !QDir( dvbConfigDir+"dvb-s" ).exists() || !QDir( dvbConfigDir+"dvb-c" ).exists() || !QDir( dvbConfigDir+"dvb-t" ).exists() ) {
+		if ( !QDir( dvbConfigDir+"dvb-s" ).exists() || !QDir( dvbConfigDir+"dvb-c" ).exists() || !QDir( dvbConfigDir+"dvb-t" ).exists() || !QDir( dvbConfigDir+"atsc" ).exists() ) {
 			if ( !localData() )
 				return false;
 		}
@@ -327,6 +343,7 @@ QStringList DVBconfig::getSourcesList( fe_type_t type )
 		case FE_QPSK : s = "dvb-s"; break;
 		case FE_QAM : s = "dvb-c"; break;
 		case FE_OFDM : s = "dvb-t"; break;
+		case FE_ATSC : s = "atsc"; break;
 		default : return list;
 	}
 	list = QDir( dvbConfigDir+s ).entryList( QDir::Files, QDir::Name );
@@ -397,6 +414,14 @@ void DVBconfig::saveDvbChanOrder( int s, int col )
 
 
 
+void DVBconfig::readFirst()
+{
+	config->setGroup( "DVB Options" );
+	probeMfe = config->readNumEntry( "ProbeMFE", 1 );
+}
+
+
+
 void DVBconfig::readConfig()
 {
 	QSize size;
@@ -416,16 +441,19 @@ void DVBconfig::readConfig()
 	beginMargin = config->readNumEntry( "BeginMargin", 5 );
 	endMargin = config->readNumEntry( "EndMargin", 10 );
 	instantDuration = config->readNumEntry( "InstantDuration", 120 );
-	format = config->readNumEntry( "PrefFormat", OutTS );
 	recordDir = config->readEntry( "RecordDir", QDir::homeDirPath() );
 	if ( !recordDir.endsWith("/") )
 		recordDir+= "/";
+	sizeFile = config->readNumEntry("SizeFile",0);
 	shiftDir = config->readEntry( "ShiftDir", QDir::homeDirPath() );
 	if ( !shiftDir.endsWith("/") )
 		shiftDir+= "/";
+	filenameFormat = config->readEntry( "filenameFormat", "%name" );
 	for ( i=0; i<(int)devList.count(); i++ ) {
 		devList.at(i)->source = config->readEntry( QString("DVB%1").arg(i), "" );
 		devList.at(i)->tuningTimeout = config->readNumEntry( QString("DVB%1_TIMEOUT").arg(i), 1500 );
+		devList.at(i)->camMaxService = config->readNumEntry( QString("DVB%1_CAM_MAX").arg(i), 1 );
+		devList.at(i)->priority = config->readNumEntry( QString("DVB%1_PRIORITY").arg(i), 10 );
 		if ( devList.at(i)->type!=FE_QPSK )
 			continue;
 		devList.at(i)->numLnb = config->readNumEntry( QString("DVB%1_NLNB").arg(i), 1 );
@@ -439,6 +467,9 @@ void DVBconfig::readConfig()
 			devList.at(i)->lnb[j].speed13v = config->readDoubleNumEntry( QString("DVB%1_LNB%2_speed13v").arg(i).arg(j), 2.5 );
 			devList.at(i)->lnb[j].speed18v = config->readDoubleNumEntry( QString("DVB%1_LNB%2_speed18v").arg(i).arg(j), 1.5 );
 		}
+		devList.at(i)->secMini = config->readNumEntry( QString("DVB%1_SEC_MINI").arg(i), 0 );
+		devList.at(i)->secTwice = config->readNumEntry( QString("DVB%1_SEC_TWICE").arg(i), 0 );
+		devList.at(i)->doS2 = config->readNumEntry( QString("DVB%1_DOS2").arg(i), 0 );
 	}
 	j = config->readNumEntry( "NumCategories", 0 );
 	for ( i=0; i<j; i++ )
@@ -455,6 +486,9 @@ void DVBconfig::readConfig()
 		devList.at(i)->usalsLatitude = usalsLatitude;
 		devList.at(i)->usalsLongitude = usalsLongitude;
 	}
+	ringBufSize = config->readNumEntry( "RingBufSize", 2 );
+	if ( ringBufSize<2 )
+		ringBufSize = 2;
 }
 
 
@@ -470,15 +504,18 @@ void DVBconfig::saveConfig()
 	config->writeEntry( "BeginMargin", beginMargin );
 	config->writeEntry( "EndMargin", endMargin );
 	config->writeEntry( "InstantDuration", instantDuration );
-	config->writeEntry( "PrefFormat", format );
 	config->writeEntry( "RecordDir", recordDir );
 	config->writeEntry( "ShiftDir", shiftDir );
+	config->writeEntry( "filenameFormat", filenameFormat );
 	config->writeEntry( "BroadcastAddress", broadcastAddress );
 	config->writeEntry( "BroadcastPort", broadcastPort );
 	config->writeEntry( "SenderPort", senderPort );
+	config->writeEntry( "ProbeMFE", probeMfe );
 	for ( i=0; i<(int)devList.count(); i++ ) {
 		config->writeEntry( QString("DVB%1").arg(i), devList.at(i)->source );
 		config->writeEntry( QString("DVB%1_TIMEOUT").arg(i), devList.at(i)->tuningTimeout );
+		config->writeEntry( QString("DVB%1_PRIORITY").arg(i), devList.at(i)->priority );
+		config->writeEntry( QString("DVB%1_CAM_MAX").arg(i), devList.at(i)->camMaxService );
 		if ( devList.at(i)->type!=FE_QPSK )
 			continue;
 		config->writeEntry( QString("DVB%1_NLNB").arg(i), devList.at(i)->numLnb );
@@ -492,6 +529,9 @@ void DVBconfig::saveConfig()
 			config->writeEntry( QString("DVB%1_LNB%2_speed13v").arg(i).arg(j), devList.at(i)->lnb[j].speed13v );
 			config->writeEntry( QString("DVB%1_LNB%2_speed18v").arg(i).arg(j), devList.at(i)->lnb[j].speed18v );
 		}
+		config->writeEntry( QString("DVB%1_SEC_MINI").arg(i), devList.at(i)->secMini );
+		config->writeEntry( QString("DVB%1_SEC_TWICE").arg(i), devList.at(i)->secTwice );
+		config->writeEntry( QString("DVB%1_DOS2").arg(i), devList.at(i)->doS2 );
 	}
 	config->writeEntry( "NumCategories", categories.count() );
 	for ( i=0; i<(int)categories.count(); i++ ) {
@@ -506,6 +546,8 @@ void DVBconfig::saveConfig()
 	config->writeEntry( "DefaultCharset", defaultCharset );
 	config->writeEntry( "UsalsLatitude", usalsLatitude );
 	config->writeEntry( "UsalsLongitude", usalsLongitude );
+	config->writeEntry( "SizeFile", sizeFile );
+	config->writeEntry( "RingBufSize", ringBufSize );
 	config->sync();
 }
 
@@ -523,7 +565,7 @@ bool DVBconfig::firstRun()
 
 
 
-DvbConfigDialog::DvbConfigDialog( DVBconfig *dc, QWidget *parent, KaffeineDvbPlugin *p ) :
+DvbConfigDialog::DvbConfigDialog( DvbPanel *pan, DVBconfig *dc, QWidget *parent, KaffeineDvbPlugin *p ) :
 	KDialogBase ( IconList, i18n("DVB Settings"), Ok|Cancel, Ok, parent, "dvbConfigDialog", true, true )
 {
 	QLabel *lab;
@@ -541,7 +583,7 @@ DvbConfigDialog::DvbConfigDialog( DVBconfig *dc, QWidget *parent, KaffeineDvbPlu
 	QSpinBox *spin;
 	KPushButton *usals;
 	QWidget *swidg;
-	QStringList rotorList; rotorList<<i18n("No rotor")<<i18n("USALS rotor")<<i18n("Positions rotor");
+	QStringList rotorList; rotorList<<i18n("No rotor")<<i18n("USALS rotor")<<i18n("Positions rotor")<<i18n("External positionner");
 
 	dvbConfig = dc;
 	timeoutSpin.setAutoDelete( true );
@@ -570,7 +612,22 @@ DvbConfigDialog::DvbConfigDialog( DVBconfig *dc, QWidget *parent, KaffeineDvbPlu
 			case FE_ATSC : dvbType->setText( i18n("Atsc") ); break;
 			default : dvbType->setText( i18n("Unknown") );
 		}
-		grid->addMultiCellWidget( dvbType, gridLine, gridLine, 1, 3 );
+		if ( dvbConfig->devList.at(i)->hasCAM ) {
+			grid->addWidget( dvbType, gridLine, 1 );
+			MCAMButton *camb = new MCAMButton( gb, i );
+			connect( camb, SIGNAL(clicked(int)), pan, SLOT(camClicked(int)) );
+			grid->addWidget( camb, gridLine, 2 );
+		}
+		else
+			grid->addMultiCellWidget( dvbType, gridLine, gridLine, 1, 3 );
+		++gridLine;
+		
+		lab = new QLabel( i18n("Tuner priority (0=Don't use):"), gb );
+		grid->addWidget( lab, gridLine, 0 );
+		spin = new QSpinBox( 0, 99, 1, gb );
+		spin->setValue( dvbConfig->devList.at(i)->priority );
+		priority.append( spin );
+		grid->addWidget( spin, gridLine, 1 );
 		++gridLine;
 
 		lab = new QLabel( i18n("Tuner timeout :"), gb );
@@ -584,6 +641,11 @@ DvbConfigDialog::DvbConfigDialog( DVBconfig *dc, QWidget *parent, KaffeineDvbPlu
 		++gridLine;
 
 		if ( dvbConfig->devList.at(i)->type==FE_QPSK ) {
+			doS2[i] = new QCheckBox( i18n("S2 capable device"), gb );
+			doS2[i]->setChecked( dvbConfig->devList.at(i)->doS2 );
+			grid->addWidget( doS2[i], gridLine, 0 );
+			++gridLine;
+			
 			lab = new QLabel( i18n("Number of LNBs:"), gb );
 			grid->addWidget( lab, gridLine, 0 );
 			satNumber[i] = new MSpinBox( gb, i );
@@ -594,6 +656,16 @@ DvbConfigDialog::DvbConfigDialog( DVBconfig *dc, QWidget *parent, KaffeineDvbPlu
 			connect( usals, SIGNAL(clicked()), this, SLOT(setUsals()));
 			grid->addWidget( usals, gridLine, 2 );
 
+			++gridLine;
+			
+			secMini[i] = new QCheckBox( i18n("Mini DiSEqC (A-B)."), gb );
+			secMini[i]->setChecked( dvbConfig->devList.at(i)->secMini );
+			secMini[i]->setEnabled( false );
+			grid->addWidget( secMini[i], gridLine, 1 );
+			secTwice[i] = new QCheckBox( i18n("Send DiSEqC commands twice."), gb );
+			secTwice[i]->setChecked( dvbConfig->devList.at(i)->secTwice );
+			grid->addWidget( secTwice[i], gridLine, 0 );
+			
 			++gridLine;
 
 			lnb0[i] = new MPushButton( gb, i, 0 );
@@ -805,10 +877,23 @@ DvbConfigDialog::DvbConfigDialog( DVBconfig *dc, QWidget *parent, KaffeineDvbPlu
 	h1->addWidget( lab );
 	grid->addLayout( h1, 4, 1 );
 
-	lab = new QLabel( i18n("Preferred format:"), gb );
+	lab = new QLabel( i18n("Max file size (0=Unlimited):"), gb );
 	grid->addWidget( lab, 5, 0 );
-	formatComb = new QComboBox( gb );
-	grid->addWidget( formatComb, 5, 1 );
+	sizeFileSpin = new QSpinBox( 1, 9999, 1, gb );
+	sizeFileSpin->setMinValue(0);
+	h1 = new QHBoxLayout();
+	h1->addWidget( sizeFileSpin );
+	lab = new QLabel( i18n("(MB)"), gb );
+	h1->addWidget( lab );
+	grid->addLayout( h1, 5, 1 );
+
+	lab = new QLabel( i18n("Filename Format:"), gb );
+	grid->addWidget( lab, 7, 0 );
+	filenameFormatLe = new QLineEdit( gb );
+	grid->addWidget( filenameFormatLe, 7, 1 );
+	helpNameBtn = new QToolButton( gb );
+	helpNameBtn->setIconSet( icon->loadIconSet("help", KIcon::Small) );
+	grid->addWidget( helpNameBtn, 7, 2 );
 
 	vb->addWidget( gb );
 	vb->addItem( new QSpacerItem( 20, 20, QSizePolicy::Ignored, QSizePolicy::Ignored ) );
@@ -818,14 +903,8 @@ DvbConfigDialog::DvbConfigDialog( DVBconfig *dc, QWidget *parent, KaffeineDvbPlu
 	beginSpin->setValue( dvbConfig->beginMargin );
 	endSpin->setValue( dvbConfig->endMargin );
 	instantDurationSpin->setValue( dvbConfig->instantDuration );
-	formatComb->insertItem( "TS" );
-	formatComb->insertItem( "MPEG_PES" );
-	formatComb->insertItem( "MPEG_PS" );
-	switch ( dvbConfig->format ) {
-		case OutTS : formatComb->setCurrentItem( 0 ); break;
-		case OutPES : formatComb->setCurrentItem( 1 ); break;
-		case OutPS : formatComb->setCurrentItem( 2 ); break;
-	}
+	sizeFileSpin->setValue( dvbConfig->sizeFile );
+	filenameFormatLe->setText( dvbConfig->filenameFormat );
 
 	page = addPage(i18n("Broadcasting"),i18n("DVB Broadcasting"),
 		KGlobal::instance()->iconLoader()->loadIcon( "network_local", KIcon::NoGroup, KIcon::SizeMedium ) );
@@ -858,9 +937,19 @@ DvbConfigDialog::DvbConfigDialog( DVBconfig *dc, QWidget *parent, KaffeineDvbPlu
 	vb = new QVBoxLayout( page, 6, 6 );
 	gb = new QGroupBox( "", page );
 	grid = new QGridLayout( gb, 1, 1, 20, 6 );
+	
+	probeMfe = new QCheckBox( i18n("Probe Multiple-Frontends (Restart required)."), gb );
+	probeMfe->setChecked( dvbConfig->probeMfe );
+	grid->addWidget( probeMfe, 0, 0 );
+	
+	lab = new QLabel( i18n("LiveShow ringbuffer size (MB) :"), gb );
+	grid->addWidget( lab, 1, 0 );
+	ringBufSize = new QSpinBox( 2, 99, 1, gb );
+	ringBufSize->setValue( dvbConfig->ringBufSize );
+	grid->addWidget( ringBufSize, 1, 1 );
 
 	lab = new QLabel( i18n("Default charset (restart needed):"), gb );
-	grid->addWidget( lab, 0, 0 );
+	grid->addWidget( lab, 2, 0 );
 	charsetComb = new QComboBox( gb );
 	charsetComb->insertItem( "ISO8859-1" );
 	charsetComb->insertItem( "ISO6937" );
@@ -868,19 +957,19 @@ DvbConfigDialog::DvbConfigDialog( DVBconfig *dc, QWidget *parent, KaffeineDvbPlu
 		charsetComb->setCurrentItem( 0 );
 	else if ( dvbConfig->defaultCharset=="ISO6937" )
 		charsetComb->setCurrentItem( 1 );
-	grid->addWidget( charsetComb, 0, 1 );
+	grid->addWidget( charsetComb, 2, 1 );
 
 	lab = new QLabel( i18n("Update scan data:"), gb );
-	grid->addWidget( lab, 1, 0 );
+	grid->addWidget( lab, 3, 0 );
 	updateBtn = new KPushButton( gb );
 	updateBtn->setGuiItem( KGuiItem(i18n("Download"), icon->loadIconSet("khtml_kget", KIcon::Small) ) );
-	grid->addWidget( updateBtn, 1, 1 );
+	grid->addWidget( updateBtn, 3, 1 );
 
 	lab = new QLabel( i18n("Dump epg's events to \n~/kaffeine_dvb_events.tx:"), gb );
-	grid->addWidget( lab, 2, 0 );
+	grid->addWidget( lab, 4, 0 );
 	dumpBtn = new KPushButton( gb );
 	dumpBtn->setGuiItem( KGuiItem(i18n("Dump"), icon->loadIconSet("filesave", KIcon::Small) ) );
-	grid->addWidget( dumpBtn, 2, 1 );
+	grid->addWidget( dumpBtn, 4, 1 );
 
 	vb->addWidget( gb );
 	vb->addItem( new QSpacerItem( 20, 20, QSizePolicy::Ignored, QSizePolicy::Ignored ) );
@@ -902,6 +991,7 @@ DvbConfigDialog::DvbConfigDialog( DVBconfig *dc, QWidget *parent, KaffeineDvbPlu
 	connect( recordDirBtn, SIGNAL(clicked()), this, SLOT(setRecordDir()) );
 	connect( shiftDirBtn, SIGNAL(clicked()), this, SLOT(setShiftDir()) );
 	connect( updateBtn, SIGNAL(clicked()), this, SLOT(downloadData()) );
+	connect( helpNameBtn, SIGNAL(clicked()), this, SLOT(fileTemplateHelp()) );
 	delete icon;
 }
 
@@ -1043,6 +1133,15 @@ void DvbConfigDialog::satNumberChanged( int value, int devNum )
 	rotor1[devNum]->setEnabled( value > 1 );
 	rotor2[devNum]->setEnabled( value > 2 );
 	rotor3[devNum]->setEnabled( value > 3 );
+
+	secMini[devNum]->setEnabled( value==2 );
+}
+
+
+
+void DvbConfigDialog::fileTemplateHelp()
+{
+	KMessageBox::information( this, i18n("Special strings are:\n- %chan (channel's name)\n- %date (the starting date, YYMMdd-hhmmss)\n- %name (the name given in timer editor or the program name from EPG)\nSo, if you set template to '%chan-%date-%name', the resulting filename will be, for example, BBC-20070919-210000-News.m2t") );
 }
 
 
@@ -1090,11 +1189,19 @@ void DvbConfigDialog::accept()
 		broadcastLe->setFocus();
 		return;
 	}
+	if ( !filenameFormatLe->text().contains("%chan") && !filenameFormatLe->text().contains("%date") && !filenameFormatLe->text().contains("%name") ) {
+		KMessageBox::sorry( this, i18n("Invalid filename format.") );
+		filenameFormatLe->setFocus();
+		return;
+	}
 
 	for ( i=0; i<(int)dvbConfig->devList.count(); i++ ) {
 		switch (dvbConfig->devList.at(i)->type) {
 		case FE_QPSK: {
 			dvbConfig->devList.at(i)->numLnb = satNumber[i]->value();
+			dvbConfig->devList.at(i)->secMini = secMini[i]->isChecked();
+			dvbConfig->devList.at(i)->secTwice = secTwice[i]->isChecked();
+			dvbConfig->devList.at(i)->doS2 = doS2[i]->isChecked();
 			if ( dvbConfig->devList.at(i)->lnb[3].rotorType==0 ) {
 				dvbConfig->devList.at(i)->lnb[3].source.clear();
 				dvbConfig->devList.at(i)->lnb[3].source.append(sat3[i]->currentText());
@@ -1132,6 +1239,7 @@ void DvbConfigDialog::accept()
 		}
 		dvbConfig->devList.at(i)->source = s;
 		dvbConfig->devList.at(i)->tuningTimeout = timeoutSpin.at(i)->value();
+		dvbConfig->devList.at(i)->priority = priority.at(i)->value();
 	}
 
 	dvbConfig->recordDir = recordDirLe->text();
@@ -1143,15 +1251,14 @@ void DvbConfigDialog::accept()
 	dvbConfig->beginMargin = beginSpin->value();
 	dvbConfig->endMargin = endSpin->value();
 	dvbConfig->instantDuration = instantDurationSpin->value();
-	switch ( formatComb->currentItem() ) {
-		case 0 : dvbConfig->format = OutTS; break;
-		case 1 : dvbConfig->format = OutPES; break;
-		case 2 : dvbConfig->format = OutPS; break;
-	}
+	dvbConfig->sizeFile = sizeFileSpin->value();
+	dvbConfig->filenameFormat = filenameFormatLe->text();
 	dvbConfig->defaultCharset = charsetComb->currentText();
 	dvbConfig->broadcastAddress = broadcastLe->text().stripWhiteSpace();
 	dvbConfig->broadcastPort = bportSpin->value();
 	dvbConfig->senderPort = sportSpin->value();
+	dvbConfig->probeMfe = probeMfe->isChecked();
+	dvbConfig->ringBufSize = ringBufSize->value();
 	dvbConfig->saveConfig();
 	done( Accepted );
 }
@@ -1398,22 +1505,24 @@ RotorConfig::RotorConfig( Device *d, DVBconfig *c, int lnb, QWidget *parent ) :
 	vb->addWidget( resetBtn );
 	vb->addItem( new QSpacerItem( 20, 20, QSizePolicy::Fixed, QSizePolicy::Fixed ) );
 
-	grid = new QGridLayout( 0, 1, 1 );
-	lab = new QLabel( i18n("13V rotor speed:"), page );
-	grid->addWidget( lab, 0, 0 );
-	speed13 = new QLineEdit( page );
-	speed13->setText( QString().setNum( dev->lnb[lnbNum].speed13v ) );
-	grid->addWidget( speed13, 0, 1 );
-	lab = new QLabel( i18n("sec./ °"), page );
-	grid->addWidget( lab, 0, 2 );
-	lab = new QLabel( i18n("18V rotor speed:"), page );
-	grid->addWidget( lab, 1, 0 );
-	speed18 = new QLineEdit( page );
-	speed18->setText( QString().setNum( dev->lnb[lnbNum].speed18v ) );
-	grid->addWidget( speed18, 1, 1 );
-	lab = new QLabel( i18n("sec./ °"), page );
-	grid->addWidget( lab, 1, 2 );
-	vb->addLayout( grid );
+	if ( dev->lnb[lnbNum].rotorType!=3 ) {
+		grid = new QGridLayout( 0, 1, 1 );
+		lab = new QLabel( i18n("13V rotor speed:"), page );
+		grid->addWidget( lab, 0, 0 );
+		speed13 = new QLineEdit( page );
+		speed13->setText( QString().setNum( dev->lnb[lnbNum].speed13v ) );
+		grid->addWidget( speed13, 0, 1 );
+		lab = new QLabel( i18n("sec./ °"), page );
+		grid->addWidget( lab, 0, 2 );
+		lab = new QLabel( i18n("18V rotor speed:"), page );
+		grid->addWidget( lab, 1, 0 );
+		speed18 = new QLineEdit( page );
+		speed18->setText( QString().setNum( dev->lnb[lnbNum].speed18v ) );
+		grid->addWidget( speed18, 1, 1 );
+		lab = new QLabel( i18n("sec./ °"), page );
+		grid->addWidget( lab, 1, 2 );
+		vb->addLayout( grid );
+	}
 
 	vb->addItem( new QSpacerItem( 20, 20, QSizePolicy::Ignored, QSizePolicy::Ignored ) );
 
@@ -1457,8 +1566,14 @@ void RotorConfig::accept()
 {
 	QListViewItem *it;
 
-	dev->lnb[lnbNum].speed18v = speed18->text().toDouble();
-	dev->lnb[lnbNum].speed13v = speed13->text().toDouble();
+	if ( dev->lnb[lnbNum].rotorType!=3 ) {
+		dev->lnb[lnbNum].speed18v = speed18->text().toDouble();
+		dev->lnb[lnbNum].speed13v = speed13->text().toDouble();
+	}
+	else {
+		dev->lnb[lnbNum].speed18v = 0;
+		dev->lnb[lnbNum].speed13v = 0;
+	}
 	dev->lnb[lnbNum].source.clear();
 	dev->lnb[lnbNum].position.clear();
 	for ( it=listView->firstChild(); it; it=it->nextSibling() ) {
