@@ -18,6 +18,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#include <fcntl.h>
+
 #include <kstandarddirs.h>
 
 #include "dvbout.h"
@@ -46,20 +48,22 @@ DVBout::DVBout( ChannelDesc chan, int anum, int tnum, KaffeineDvbPlugin *p )
 	}
 
 	fdPipe=0;
-	outType = 0;
 	channel = chan;
 	thWrite = 0;
-	tp = 0;
-	tpShift = 0;
 	rtp = 0;
-	activeApid = 0;
+	if ( channel.vpid )
+		pids.append( channel.vpid );
+	for ( i=0; i<channel.napid && i<MAX_AUDIO; i++ )
+		pids.append( channel.apid[i].pid );
+	for ( i=0; i<channel.nsubpid && i<MAX_DVBSUB; i++ )
+		pids.append( channel.subpid[i].pid );
 	wBuf = NULL;
 	timeShifting = beginLive = false;
 	haveRec = haveLive = instantRec = haveBroadcast = false;
 	patpmt = wpatpmt = false;
 	connect( &stopRecTimer, SIGNAL(timeout()), this, SLOT(stopRec()) );
 	connect( &timerPatPmt, SIGNAL(timeout()), this, SLOT(setPatPmt()) );
-	if ( channel.apid[activeApid].pid!=8192)
+	if ( !pids.contains(8192) )
 		timerPatPmt.start(500);
 }
 
@@ -105,7 +109,7 @@ void DVBout::writePat()
 	tspat[0x0d] = tspat[0x0e] = 0x00; tspat[0x0f] = 0xe0; tspat[0x10] = 0x10;
 	// Program Map PID
 	pmtpid = 0xff;
-	while ( pmtpid==channel.vpid || pmtpid==channel.apid[activeApid].pid || pmtpid==channel.subpid[0].pid ) pmtpid--;
+	while ( pids.contains( pmtpid ) ) pmtpid--;
 	tspat[0x11] = 0x03; tspat[0x12] = 0xe8; tspat[0x13] = 0xe0; tspat[0x14] = pmtpid;
 	// Put CRC in ts[0x15...0x18]
 	calculateCRC( tspat + 0x05, tspat + 0x15 );
@@ -132,65 +136,80 @@ void DVBout::writePmt()
 	tspmt[0x0a] = 0xc1;
 	// section # and last section #
 	tspmt[0x0b] = tspmt[0x0c] = 0x00;
-	// PCR PID
-	tspmt[0x0d] = channel.vpid>>8; tspmt[0x0e] = channel.vpid&0xff;
+	if ( channel.vpid ) {
+		// PCR PID
+		tspmt[0x0d] = channel.vpid>>8; tspmt[0x0e] = channel.vpid&0xff;
+	}
+	else if ( channel.napid )
+		tspmt[0x0d] = channel.apid[0].pid>>8; tspmt[0x0e] = channel.apid[0].pid&0xff;
 	// program_info_length == 0
 	tspmt[0x0f] = 0xf0; tspmt[0x10] = 0x00;
-	// Program Map / Video PID
-	tspmt[0x11] = channel.vType; // video stream type
-	tspmt[0x12] = channel.vpid>>8; tspmt[0x13] = channel.vpid&0xff;
-	tspmt[0x14] = 0xf0; tspmt[0x15] = 0x09; // es info length
-	// useless info
-	tspmt[0x16] = 0x07; tspmt[0x17] = 0x04; tspmt[0x18] = 0x08; tspmt[0x19] = 0x80;
-	tspmt[0x1a] = 0x24; tspmt[0x1b] = 0x02; tspmt[0x1c] = 0x11; tspmt[0x1d] = 0x01;
-	tspmt[0x1e] = 0xfe;
-	off = 0x1e;
-	// audio pid
-	if ( channel.apid[activeApid].ac3 ) {
-		tspmt[++off] = 0x81; // stream type = xine see this as ac3
-		tspmt[++off] = channel.apid[activeApid].pid>>8; tspmt[++off] = channel.apid[activeApid].pid&0xff;
-		tspmt[++off] = 0x00; tspmt[++off] = 0x0c; // es info length
-		tspmt[++off] = 0x05; tspmt[++off] = 0x04; tspmt[++off] = 0x41;
-		tspmt[++off] = 0x43; tspmt[++off] = 0x2d; tspmt[++off] = 0x33;
+	if ( channel.vpid ) {
+		// Program Map / Video PID
+		tspmt[0x11] = channel.vType; // video stream type
+		tspmt[0x12] = channel.vpid>>8; tspmt[0x13] = channel.vpid&0xff;
+		tspmt[0x14] = 0xf0; tspmt[0x15] = 0x00; // es info length
+		off = 0x15;
 	}
-	else {
-		tspmt[++off] = 0x04; // stream type = audio
-		tspmt[++off] = channel.apid[activeApid].pid>>8; tspmt[++off] = channel.apid[activeApid].pid&0xff;
-		tspmt[++off] = 0xf0; tspmt[++off] = 0x06; // es info length
+	else
+		off = 0x10;
+	// audio pids
+	i = 0;
+	for ( i=0; i<channel.napid && i<MAX_AUDIO; i++ ) {
+		if ( channel.apid[i].ac3 ) {
+			tspmt[++off] = 0x81; // stream type = xine see this as ac3
+			tspmt[++off] = channel.apid[i].pid>>8; tspmt[++off] = channel.apid[i].pid&0xff;
+			tspmt[++off] = 0x00; tspmt[++off] = 0x0c; // es info length
+			tspmt[++off] = 0x05; tspmt[++off] = 0x04; tspmt[++off] = 0x41;
+			tspmt[++off] = 0x43; tspmt[++off] = 0x2d; tspmt[++off] = 0x33;
+		}
+		else {
+			tspmt[++off] = 0x04; // stream type = audio
+			tspmt[++off] = channel.apid[i].pid>>8; tspmt[++off] = channel.apid[i].pid&0xff;
+			tspmt[++off] = 0xf0; tspmt[++off] = 0x06; // es info length
+		}
+		tspmt[++off] = 0x0a; // iso639 descriptor tag
+		tspmt[++off] = 0x04; // descriptor length
+		if ( !channel.apid[i].lang.isEmpty() ) {
+			tspmt[++off] = channel.apid[i].lang.constref(0);
+			tspmt[++off] = channel.apid[i].lang.constref(1);
+			if ( channel.apid[i].ac3 )
+				tspmt[++off] = '_';
+			else
+				tspmt[++off] = channel.apid[i].lang.constref(2);
+		}
+		else if ( channel.apid[i].ac3 ) {
+			tspmt[++off] = 'd';
+			tspmt[++off] = 'd';
+			tspmt[++off] = 49+i;
+		}
+		else {
+			tspmt[++off] = 'c';
+			tspmt[++off] = 'h';
+			tspmt[++off] = 49+i;
+		}
+		tspmt[++off] = 0x00; // audio type
 	}
-	tspmt[++off] = 0x0a; // iso639 descriptor tag
-	tspmt[++off] = 0x04; // descriptor length
-	if ( !channel.apid[activeApid].lang.isEmpty() ) {
-		tspmt[++off] = channel.apid[activeApid].lang.constref(0);
-		tspmt[++off] = channel.apid[activeApid].lang.constref(1);
-		tspmt[++off] = channel.apid[activeApid].lang.constref(2);
-	}
-	else {
-		tspmt[++off] = '?';
-		tspmt[++off] = '?';
-		tspmt[++off] = '?';
-	}
-	tspmt[++off] = 0x00; // audio type
 	// Subtitles
-	if ( channel.subpid[0].pid ) {
+	for ( i=0; i<channel.nsubpid && i<MAX_DVBSUB; i++ ) {
 		tspmt[++off] = 0x06; // stream type = ISO_13818_PES_PRIVATE
-		tspmt[++off] = channel.subpid[0].pid>>8; tspmt[++off] = channel.subpid[0].pid&0xff;
+		tspmt[++off] = channel.subpid[i].pid>>8; tspmt[++off] = channel.subpid[i].pid&0xff;
 		tspmt[++off] = 0xf0; tspmt[++off] = 0x0a; // es info length
 		tspmt[++off] = 0x59; //DVB sub tag
 		tspmt[++off] = 0x08; // descriptor length
-		if ( !channel.subpid[0].lang.isEmpty() ) {
-			tspmt[++off] = channel.subpid[0].lang.constref(0);
-			tspmt[++off] = channel.subpid[0].lang.constref(1);
-			tspmt[++off] = channel.subpid[0].lang.constref(2);
+		if ( !channel.subpid[i].lang.isEmpty() ) {
+			tspmt[++off] = channel.subpid[i].lang.constref(0);
+			tspmt[++off] = channel.subpid[i].lang.constref(1);
+			tspmt[++off] = channel.subpid[i].lang.constref(2);
 		}
 		else {
-			tspmt[++off] = '?';
-			tspmt[++off] = '?';
-			tspmt[++off] = '?';
+			tspmt[++off] = 'c';
+			tspmt[++off] = 'h';
+			tspmt[++off] = 49+i;
 		}
-		tspmt[++off] = channel.subpid[0].type; //sub type
-		tspmt[++off] = channel.subpid[0].page>>8; tspmt[++off] = channel.subpid[0].page&0xff; // comp_page_id
-		tspmt[++off] = channel.subpid[0].id>>8; tspmt[++off] = channel.subpid[0].id&0xff; // anc_page_id
+		tspmt[++off] = channel.subpid[i].type; //sub type
+		tspmt[++off] = channel.subpid[i].page>>8; tspmt[++off] = channel.subpid[i].page&0xff; // comp_page_id
+		tspmt[++off] = channel.subpid[i].id>>8; tspmt[++off] = channel.subpid[i].id&0xff; // anc_page_id
 	}
 	tspmt[0x07] = off-3; // update section_length
 	// Put CRC in ts[0x29...0x2c]
@@ -201,13 +220,6 @@ void DVBout::writePmt()
 
 
 
-int DVBout::currentAudioPid() const
-{
-	return activeApid;
-}
-
-
-
 bool DVBout::hasInstantRec() const
 {
 	return instantRec;
@@ -250,20 +262,19 @@ bool DVBout::doPause( const QString &name ) // called from dvbstream::run()
 		liveFile.writeBlock( (char*)tspmt, TS_SIZE );
 		mutex.lock();
 		haveLive = false;
-		mutex.unlock();
-		if ( !wait(100) ) {
+		if ( !wait(1000) ) {
 			terminate();
 			wait();
 		}
-		mutex.lock();
-		haveLive = true;
 		if ( close( fdPipe )<0 )
 			perror("close out pipe: ");
 		else
 			fprintf(stderr,"out pipe closed\n");
 		fdPipe = 0;
-		mutex.unlock();
+		if ( wDist>0 )
+			liveFile.writeBlock( (char*)(wBuf+(wRead*TS_SIZE*NTS)), TS_SIZE*NTS*wDist );
 		timeShifting = true;
+		mutex.unlock();
 		//emit shifting( timeShifting );
 	}
 	return true;
@@ -278,20 +289,21 @@ void DVBout::setPatPmt()
 
 
 
-bool DVBout::goLive( const QString &name, int napid )
+bool DVBout::goLive( const QString &name, int ringBufSize )
 {
 	if ( fdPipe ) return false;
 
 	haveLive = true;
 	pipeName = name;
 	beginLive = true;
-	activeApid = napid;
+	//activeApid = napid;
 
 	writePat();
 	writePmt();
-	if ( channel.apid[activeApid].pid!=8192 )
+	if ( !pids.contains(8192) )
 		patpmt = wpatpmt = true;
-	wBuf = new unsigned char[TS_SIZE*NTS*100];
+	wbufSize = ringBufSize*1024*1024/(TS_SIZE*NTS);
+	wBuf = new unsigned char[TS_SIZE*NTS*wbufSize];
 	if ( !wBuf ) fprintf( stderr, "\nNO WBUF !!!\n\n" );
 	wRead = wWrite = wDist = 0;
 	start();
@@ -318,7 +330,7 @@ void DVBout::stopLive()
 		emit shifting( timeShifting );
 	}
 	mutex.unlock();
-	if ( !wait(500) ) {
+	if ( !wait(1000) ) {
 		terminate();
 		wait();
 	}
@@ -356,62 +368,38 @@ void DVBout::stopBroadcast()
 
 
 
-bool DVBout::goRec( int type, const QString &name, int maxsize, RecTimer *t )
+bool DVBout::goRec( const QString &name, int maxsize, RecTimer *t )
 {
-	int ps = 0;
-	bool ok;
 	QString fname=name;
 
+	if ( haveRec )
+		return false;
+
 	recTimer = t;
+	fileName=name;
+	fileNumber=0;
 
-	if ( outType ) return false;
-	if ( tp ) return false;
-
-	if ( channel.apid[activeApid].ac3 || !channel.vpid || channel.subpid[0].pid || channel.apid[activeApid].pid==8192 )
-		type = OutTS;
-
-	switch ( type ) {
-		case OutPS : ps = 1;
-		case OutPES : {
-			if ( QFile(fname+".mpg").exists() )
-				renameFile( fname, ".mpg" );
-			tp = new Ts2Pes( fname, channel.apid[0].pid, channel.vpid, ps, &ok );
-			if ( !ok ) {
-				delete tp;
-				tp = 0;
-				return false;
-			}
-			tp->go();
-			recTimer->fullPath = fname+".mpg";
-			break;
-		}
-		case OutTS : {
-			fileName=name;
-			fileNumber=0;
-
-			if (maxsize>0) {
-				fileMaxSize = (long long int)1048576*(long long int)maxsize;
-				fname=fileName+"_"+QString().setNum(fileNumber);
-			}
-			else {
-				fileMaxSize = 0;
-				fname=fileName;
-			}
-			if ( QFile(fname+".m2t").exists() )
-				renameFile( fname, ".m2t" );
-			writePat();
-			writePmt();
-			if ( channel.apid[activeApid].pid!=8192 )
-				patpmt = true;
-			outFile.setName( fname+".m2t" );
-			if ( !outFile.open( IO_WriteOnly | IO_Truncate ) ) return false;
-			outFile.writeBlock( (char*)tspat, TS_SIZE );
-			outFile.writeBlock( (char*)tspmt, TS_SIZE );
-			recTimer->fullPath = fname+".m2t";
-			break;
-		}
+	if (maxsize>0) {
+		fileMaxSize = (long long int)1048576*(long long int)maxsize;
+		fname=fileName+"_"+QString().setNum(fileNumber);
 	}
-	outType = type;
+	else {
+		fileMaxSize = 0;
+		fname=fileName;
+	}
+	if ( QFile(fname+".m2t").exists() )
+		renameFile( fname, ".m2t" );
+	writePat();
+	writePmt();
+	if ( channel.apid[0].pid!=8192 )
+		patpmt = true;
+	outFile.setName( fname+".m2t" );
+	if ( !outFile.open( IO_WriteOnly | IO_Truncate ) )
+		return false;
+	outFile.writeBlock( (char*)tspat, TS_SIZE );
+	outFile.writeBlock( (char*)tspmt, TS_SIZE );
+	recTimer->fullPath = fname+".m2t";
+
 	haveRec = true;
 	if ( recTimer ) {
 		QTime t = recTimer->duration.addSecs( QDateTime::currentDateTime().secsTo(recTimer->begin) );
@@ -447,21 +435,13 @@ void DVBout::changeTimer( int ms )
 
 void DVBout::stopRec()
 {
-	if ( !(outType) ) return;
+	if ( !haveRec )
+		return;
 
 	if ( stopRecTimer.isActive() )
 		stopRecTimer.stop();
 	mutex.lock();
-	outType=0;
-	switch (outType) {
-		case OutPES :
-		case OutPS : {
-			delete tp;
-			tp = 0;
-			break;
-		}
-		case OutTS : outFile.close();
-	}
+	outFile.close();
 	mutex.unlock();
 	haveRec = instantRec = false;
 	fprintf( stderr, "Recording stopped: %s\n", channel.name.latin1() );
@@ -481,7 +461,7 @@ void DVBout::process( unsigned char *buf, int size )
 
 	for ( i=0; i<size; i+=TS_SIZE ) {
 		pid = (((buffer[1] & 0x1f) << 8) | buffer[2]);
-		if ( channel.vpid==pid || channel.apid[activeApid].pid==pid || channel.subpid[0].pid==pid || channel.apid[activeApid].pid==8192) {
+		if ( pids.contains( pid ) || pids.contains( 8192) ) {
 			memcpy( thBuf+thWrite, buffer, TS_SIZE );
 			thWrite+=TS_SIZE;
 			if ( thWrite==(TS_SIZE*NTS ) ) {
@@ -493,14 +473,16 @@ void DVBout::process( unsigned char *buf, int size )
 						beginLive = !beginLive;
 						start();
 					}
-					if ( wDist<95 ) {
+					if ( wDist<wbufSize ) {
 						memcpy( wBuf+(wWrite*TS_SIZE*NTS), thBuf, TS_SIZE*NTS );
 						wpatpmt = patpmt;
 						++wDist;
 						++wWrite;
-						if ( wWrite>99 )
+						if ( wWrite==wbufSize )
 							wWrite = 0;
-						//fprintf(stderr,"WDIST = %d\n",wDist);
+					}
+					else {
+						fprintf(stderr,"Live ringbuffer full!! (%d)\n",wDist);
 					}
 				}
 				else if ( timeShifting ) {
@@ -510,11 +492,8 @@ void DVBout::process( unsigned char *buf, int size )
 					}
 					liveFile.writeBlock( (char*)thBuf, TS_SIZE*NTS );
 				}
-				if ( outType > OutTS )
-					tp->run( thBuf, TS_SIZE*NTS );
-				else if ( outType==OutTS && fileNumber>=0) {
-					if ((fileMaxSize>0)&&(outFile.size()>=fileMaxSize))
-					{
+				if ( haveRec && fileNumber>=0 ) {
+					if ((fileMaxSize>0)&&(outFile.size()>=fileMaxSize)) {
 						outFile.close();
 						fileNumber++;
 						fname=fileName+"_"+QString().setNum(fileNumber);
@@ -558,7 +537,7 @@ void DVBout::run()
 {
 	if ( haveLive && fdPipe ) {
 		while ( haveLive && fdPipe ) {
-			if ( wDist>0 ) {
+			if ( wDist>5 ) {
 				if ( wpatpmt ) {
 					write( fdPipe, tspat, TS_SIZE );
 					write( fdPipe, tspmt, TS_SIZE );
@@ -567,11 +546,12 @@ void DVBout::run()
 				write( fdPipe, wBuf+(wRead*TS_SIZE*NTS), TS_SIZE*NTS );
 				--wDist;
 				++wRead;
-				if ( wRead>99 )
+				if ( wRead==wbufSize )
 					wRead = 0;
 			}
-			else
+			else {
 				usleep( 100 );
+			}
 		}
 		return;
 	}
