@@ -45,7 +45,6 @@
 #include <kxmlguifactory.h>
 #include <kparts/componentfactory.h>
 
-#include "kaffeineinput.h"
 #include "kaffeinedvbplugin.h"
 #include "dvbpanel.h"
 #include "channeldesc.h"
@@ -56,6 +55,12 @@
 
 #define CHANICONSIZE 28
 
+#define MODE_FREE 0
+#define MODE_LIVE 100
+#define MODE_BROADCAST 200
+#define MODE_RECORD 300
+#define MODE_CANTDO 400
+
 
 
 DIconViewItem::DIconViewItem( DvbPanel *pan, QIconView *parent, const QString &text, const QPixmap &icon )
@@ -180,6 +185,7 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	//channelsCb->setPaletteBackgroundColor( QColor(255,255,200) );
 	connect( channelsCb, SIGNAL(itemRenamed(QListViewItem*)), this, SLOT(channelNumberChanged(QListViewItem*)) );
 	connect( channelsCb, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(channelSelected(QListViewItem*)) );
+	connect( channelsCb, SIGNAL(clicked(QListViewItem*)), this, SLOT(channelClicked(QListViewItem*)) );
 	connect( channelsCb, SIGNAL(contextMenuRequested(QListViewItem*,const QPoint&,int)), this, SLOT(channelPopup(QListViewItem*,const QPoint&,int)) );
 	channelsCb->setItemsRenameable( true );
 	channelsCb->addColumn( i18n("Number") );
@@ -191,17 +197,9 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	vb->addWidget( channelsCb );
 
 	h1 = new QHBoxLayout();
-	//QLabel *lab = new QLabel( i18n("Audio :"), panel );
-	//lab->resize( lab->sizeHint() );
-	//lab->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
-	//h1->addWidget( lab );
-	audioComb = new QComboBox( panel );
-	QToolTip::add( audioComb, i18n("Audio Channels") );
-	audioComb->setSizePolicy( QSizePolicy (QSizePolicy::Preferred, QSizePolicy::Fixed) );
-	audioComb->setEnabled( false );
-	h1->addWidget( audioComb );
 	recordBtn = new QToolButton( panel );
 	QToolTip::add( recordBtn, i18n("Instant Record") );
+	recordBtn->setToggleButton( true );
 	h1->addWidget( recordBtn );
 	broadcastBtn = new QToolButton( panel );
 	QToolTip::add( broadcastBtn, i18n("Broadcast") );
@@ -240,8 +238,11 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	tvcPix = icon->loadIcon( "kdvbtvc", KIcon::Small );
 	raPix = icon->loadIcon( "kdvbra", KIcon::Small );
 	racPix = icon->loadIcon( "kdvbrac", KIcon::Small );
+	QIconSet iconset;
+	iconset.setPixmap( icon->loadIcon( "filesave", KIcon::Small ), QIconSet::Small );
+	iconset.setPixmap( icon->loadIcon( "player_record", KIcon::Small), QIconSet::Small, QIconSet::Normal, QIconSet::On );
+	recordBtn->setIconSet( iconset );
 	broadcastBtn->setIconSet( icon->loadIconSet("network_local", KIcon::Small) );
-	recordBtn->setIconSet( icon->loadIconSet("filesave", KIcon::Small) );
 	dateBtn->setIconSet( icon->loadIconSet("date", KIcon::Small) );
 	infoBtn->setIconSet( icon->loadIconSet("view_text", KIcon::Small) );
 	osdBtn->setIconSet( icon->loadIconSet("info", KIcon::Small) );
@@ -253,7 +254,6 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	setXMLFile("kaffeinedvb.rc");
 	setupActions();
 
-	connect( audioComb, SIGNAL(activated(int)), this, SLOT(audioSelected(int)) );
 	connect( this, SIGNAL(zap(ChannelDesc*)), SLOT(dvbZap(ChannelDesc*)) );
 	connect( configBtn, SIGNAL(clicked()), this, SLOT(showConfigDialog()));
 	connect( recallBtn, SIGNAL(clicked()), this, SLOT(recallZap()));
@@ -714,6 +714,40 @@ void DvbPanel::dumpEvents()
 
 
 
+void DvbPanel::channelClicked( QListViewItem *it )
+{
+	QPtrListIterator<ChannelDesc> iter( channels );
+	ChannelDesc *itdesc, *desc=0;
+	DvbStream *d = 0;
+
+	if ( !it )
+		return;
+
+	iter.toFirst();
+	while ( (itdesc=iter.current())!=0 ) {
+		if ( itdesc->name==it->text(1) ) {
+			desc = itdesc;
+			break;
+		}
+		++iter;
+	}
+
+	if ( !desc )
+		return;
+
+	for (int j = 0; j < (int)dvb.count(); j ++) {
+		if (dvb.at(j)->canSource(desc)) {
+			d = dvb.at(j);
+			break;
+		}
+	}
+
+	osdMode = 0;
+	dvbOSD( *desc, d );
+}
+
+
+
 void DvbPanel::resetOSD()
 {
 	osdMode = 0;
@@ -906,7 +940,9 @@ void DvbPanel::dvbOSD(ChannelDesc liveChannel, DvbStream *d, int timeShift /* =
 	k = 0;
 
 	esrc = events.getEventSource( liveChannel.tp.source );
-	es = esrc->getEventSid( liveChannel.tp.tsid, liveChannel.sid );
+	es = esrc->getEventSid( liveChannel.tp.nid, liveChannel.tp.tsid, liveChannel.sid );
+	if ( !es )
+		goto end;
 	es->lock();
 	esEvents = es->getEvents();
 
@@ -958,6 +994,7 @@ void DvbPanel::dvbOSD(ChannelDesc liveChannel, DvbStream *d, int timeShift /* =
 	}
 	es->unlock();
 
+end:
 	if ( osdMode==0 )
 		osdTimer.start( 5000, true );
 	osdMode++;
@@ -1059,7 +1096,7 @@ void DvbPanel::setConfig()
 
 void DvbPanel::showConfigDialog()
 {
-	int ret;
+	int i, ret;
 
 loop:
 	if ( !dvbConfig->haveData() ) {
@@ -1072,7 +1109,9 @@ loop:
 		return;
 	}
 
-	DvbConfigDialog dlg( dvbConfig, mainWidget, plug );
+	for ( i=0; i<(int)dvb.count(); i++ )
+		dvb.at(i)->probeCam();
+	DvbConfigDialog dlg( this, dvbConfig, mainWidget, plug );
 	connect( dlg.dumpBtn, SIGNAL(clicked()), this, SLOT(dumpEvents()) );
 	ret = dlg.exec();
 	disconnect( dlg.dumpBtn, SIGNAL(clicked()), this, SLOT(dumpEvents()) );
@@ -1080,6 +1119,14 @@ loop:
 		return;
 	rtp->setSocket( dvbConfig->broadcastAddress, dvbConfig->broadcastPort, dvbConfig->senderPort );
 	cleaner->setPaths( dvbConfig->shiftDir, dvbConfig->recordDir );
+	fillChannelList();
+}
+
+
+
+void DvbPanel::camClicked( int devNum )
+{
+	dvb.at(devNum)->showCamDialog();
 }
 
 
@@ -1092,6 +1139,8 @@ QPtrList<Transponder> DvbPanel::getSourcesStatus()
 	Transponder t;
 
 	for ( i=0; i<(int)dvb.count(); i++ ) {
+		if ( !dvb.at(i)->getPriority() ) // priority==0==don't use
+			continue;
 		list = dvb.at(i)->getSources();
 		for ( j=0; j<(int)list.count(); j++ ) {
 			if ( dvb.at(i)->hasRec() || dvb.at(i)->hasBroadcast() )
@@ -1144,7 +1193,7 @@ void DvbPanel::fillChannelList( ChannelDesc *ch )
 		}
 		else if ( currentCategory!="All" && chan->category!=currentCategory )
 			continue;
-		it = new KListViewItem( channelsCb, QString().sprintf("%04d", chan->num), chan->name, chan->tp.source );
+		it = new KListViewItem( channelsCb, QString().sprintf("%05d", chan->num), chan->name, chan->tp.source );
 		if ( ch && ch==chan )
 			visible = it;
 		it->setDragEnabled( true );
@@ -1179,15 +1228,15 @@ void DvbPanel::fillChannelList( ChannelDesc *ch )
 DvbStream* DvbPanel::getWorkingDvb( int mode, ChannelDesc *chan )
 {
 	int i, ret;
-	QValueList<int> working; //  notTuned=0, hasLive=1, hasBroadcast=2, hasRec=3, can'tDoChannel=4
+	QValueList<int> working; //  free=0, hasLive=100, hasBroadcast=200, hasRec=300, can'tDoChannel=400
 
 	for ( i=0; i<(int)dvb.count(); i++ )
-		working.append( 0 );
+		working.append( 100-dvb.at(i)->getPriority() );
 
 	// fill in working status
 	for ( i=0; i<(int)dvb.count(); i++ ) {
-		if ( !dvb.at(i)->canSource( chan ) ) {
-			working[i] = 4;
+		if ( !dvb.at(i)->canSource( chan ) || working[i]==100 ) {
+			working[i] = MODE_CANTDO;
 			continue;
 		}
 		if ( dvb.at(i)->isTuned() ) {
@@ -1195,14 +1244,18 @@ DvbStream* DvbPanel::getWorkingDvb( int mode, ChannelDesc *chan )
 				return dvb.at(i); // use that one since it's already tuned on the good mplex
 			}
 			else if ( dvb.at(i)->hasRec() )
-				working[i] = 3;
+				working[i] += MODE_RECORD;
 			else if ( dvb.at(i)->hasBroadcast() )
-				working[i] = 2;
-			else
-				working[i] = 1;
+				working[i] += MODE_BROADCAST;
+			else {
+				if ( mode==MODE_LIVE )
+					working[i] += MODE_FREE;
+				else
+					working[i] += MODE_LIVE;
+			}
 		}
 		else
-			working[i] = 0;
+			working[i] += MODE_FREE;
 	}
 	ret = 0;
 	// search for least working card
@@ -1240,7 +1293,7 @@ void DvbPanel::setBroadcast()
 		return;
 	}
 
-	d = getWorkingDvb( 2, list.at(0) );
+	d = getWorkingDvb( MODE_BROADCAST, list.at(0) );
 
 	if ( d )
 		ret = d->canStartBroadcast( live, list.at(0) );
@@ -1284,7 +1337,7 @@ void DvbPanel::checkTimers()
 			}
 			if ( !chan )
 				continue;
-			d = getWorkingDvb( 3, chan );
+			d = getWorkingDvb( MODE_RECORD, chan );
 			live = false;
 			if ( d )
 				ret = d->canStartTimer( live, chan );
@@ -1295,7 +1348,7 @@ void DvbPanel::checkTimers()
 					stopLive();
 					emit dvbStop();
 				}
-				if ( d->startTimer( chan, dvbConfig->recordDir, dvbConfig->sizeFile, t ) ) {
+				if ( d->startTimer( chan, dvbConfig->recordDir, dvbConfig->sizeFile, t, dvbConfig->filenameFormat ) ) {
 					KProcess proc;
 					proc << QDir::homeDirPath()+"/bin/kaffeine_recording";
 					proc << "On";
@@ -1304,7 +1357,6 @@ void DvbPanel::checkTimers()
 					if ( timersDialog )
 						emit timersChanged();
 					saveTimerList();
-					audioComb->setEnabled( false );
 					i--;
 				}
 				else
@@ -1343,33 +1395,39 @@ void DvbPanel::setRecord()
 		}
 	}
 	if ( !d ) {
+		recordBtn->setOn( false );
 		showTimers();
 		return;
 	}
 
-	desc = events.getEventDesc( curchan.tp.source, curchan.tp.tsid, curchan.sid, 0 );
+	desc = events.getEventDesc( curchan.tp.source, curchan.tp.nid, curchan.tp.tsid, curchan.sid, 0 );
+
 	if ( desc )
 		s = desc->title;
 
-	if ( s.isEmpty() )
-		s = curchan.name+"-"+QDateTime::currentDateTime().toString("yyyyMMdd-hhmmss");
+	if ( s.isEmpty() ) {
+		if ( !dvbConfig->filenameFormat.contains("%chan") )
+			s = curchan.name;
+		if ( !dvbConfig->filenameFormat.contains("%date") )
+			s+="_"+QDateTime::currentDateTime().toString( "yyyyMMdd-hhmmss" );
+	}
 
 	rt = new RecTimer();
-	rt->name = s.replace( "/", "_" ).replace( ">", "_" ).replace("<","_").replace(":","_").replace('"',"_").replace("\\","_").replace("|","_");
+	rt->name = s;
 	rt->channel = curchan.name;
 	rt->begin = QDateTime::currentDateTime();
 	rt->duration = QTime( 0,0,0).addSecs( dvbConfig->instantDuration*60 ) ;
-	rt->filetype = dvbConfig->format;
 	rt->running = 1;
 	rt->mode = 0;
 
 	ret = d->canStartTimer( live, &curchan );
 	if ( ret==0 ) {
-		if ( d->startTimer( &curchan, dvbConfig->recordDir, dvbConfig->sizeFile, rt ) ) {
+		if ( d->startTimer( &curchan, dvbConfig->recordDir, dvbConfig->sizeFile, rt, dvbConfig->filenameFormat ) ) {
 			KProcess proc;
 			proc << QDir::homeDirPath()+"/bin/kaffeine_recording";
 			proc << "On";
 			proc.start( KProcess::DontCare );
+
 			for ( i=0; i<(int)timers.count(); i++ ) {
 				t = timers.at(i);
 				if ( rt->begin>t->begin )
@@ -1379,16 +1437,24 @@ void DvbPanel::setRecord()
 			if ( timersDialog )
 				emit timersChanged();
 			saveTimerList();
-			audioComb->setEnabled( false );
-			KMessageBox::information( mainWidget, i18n("Timer successfully created and started.") );
+			emit showOSD( i18n("Instant Record successfully started"), 5000, 3 );
 		}
 		else {
-			fprintf( stderr, "start timer failed!\n" );
+			KMessageBox::information( mainWidget, i18n("Instant Recording failed to start.") );
+			fprintf( stderr, "DvbPanel::setRecord:start timer failed!\n" );
 			delete rt;
 		}
 	}
 	else {
 		delete rt;
+		for ( i=0; i<(int)timers.count(); i++ ) {
+			t = timers.at(i);
+			if ( d->liveIsRecording() && t->running && curchan.name == t->channel ) {
+				d->updateTimer( t, 0 );
+				emit showOSD( i18n("Recording successfully stopped"), 5000, 3 );
+				return;
+			}
+		}
 		showTimers();
 	}
 }
@@ -1417,6 +1483,14 @@ void DvbPanel::setRecordLed( bool on )
 			recordLed->on();
 		fillChannelList();
 	}
+
+	for ( i=0; i<(int)dvb.count(); i++ ) {
+		if ( dvb.at(i)->liveIsRecording() ) {
+			recordBtn->setOn( true );
+			return;
+		}
+	}
+	recordBtn->setOn( false );
 }
 
 
@@ -1460,6 +1534,7 @@ void DvbPanel::killTimer( RecTimer *t )
 				proc << "Off";
 			proc << KProcess::quote( t->fullPath );
 			proc.start( KProcess::DontCare );
+
 			if ( t->mode )
 				updateModeTimer( t );
 			else
@@ -1499,7 +1574,6 @@ void DvbPanel::updateModeTimer( RecTimer *t )
 	nt->channel = t->channel;
 	nt->begin = t->begin;
 	nt->duration = t->duration;
-	nt->filetype = t->filetype;
 	nt->running = 0;
 	nt->mode = t->mode;
 	timers.remove( t );
@@ -1530,7 +1604,7 @@ void DvbPanel::showTimers()
         for ( it = map.begin(); it != end; ++it )
         	list.append( it.data() );
 
-	timersDialog = new KRecord( list, &timers, mainWidget, dvbConfig->timerSize, dvbConfig->format );
+	timersDialog = new KRecord( list, &timers, mainWidget, dvbConfig->timerSize );
 	for ( i=0; i<(int)dvb.count(); i++ ) {
 		connect( timersDialog, SIGNAL(updateTimer(RecTimer*,int)), dvb.at(i), SLOT(updateTimer(RecTimer*,int)) );
 	}
@@ -1573,7 +1647,6 @@ void DvbPanel::newTimer( QString channel, QString name, QDateTime begin, QTime d
 	rt->channel = channel;
 	rt->begin = begin.addSecs( -(dvbConfig->beginMargin*60) );
 	rt->duration = duration.addSecs( (dvbConfig->beginMargin+dvbConfig->endMargin)*60 ) ;
-	rt->filetype = dvbConfig->format;
 	rt->running = 0;
 	rt->mode = 0;
 
@@ -1634,7 +1707,6 @@ void DvbPanel::setShiftLed( bool on )
 {
 	if ( on ) {
 		shiftLed->on();
-		audioComb->setEnabled( false );
 		diskTimer.start( 300000 );
 	}
 	else {
@@ -1682,30 +1754,6 @@ void DvbPanel::channelSelected( QListViewItem *it )
 
 
 
-void DvbPanel::audioSelected( int n )
-{
-	int i;
-	DvbStream *d=0;
-
-	for ( i=0; i<(int)dvb.count(); i++ ) {
-		if ( dvb.at(i)->hasLive() ) {
-			d = dvb.at(i);
-			break;
-		}
-	}
-	if ( !d )
-		return;
-
-	ChannelDesc chan=d->getLiveChannel();
-	emit setTimeShiftFilename( "" );
-	d->preStopLive();
-	emit dvbPause( false );
-	d->stopLive( &chan );
-	finalZap( d, &chan, false, n );
-}
-
-
-
 bool DvbPanel::nextTrack( MRL& )
 {
 	next();
@@ -1740,6 +1788,7 @@ bool DvbPanel::trackNumber( int num, MRL& )
 
 void DvbPanel::recallZap()
 {
+	emit showMe( this );
 	playNumChannel( recallChannel );
 }
 
@@ -1798,7 +1847,7 @@ void DvbPanel::playLastChannel()
 		QTimer::singleShot( 2000, this, SLOT(playLastChannel()) );
 		return;
 	}
-	d = getWorkingDvb( 2, list.at(0) );
+	d = getWorkingDvb( MODE_BROADCAST, list.at(0) );
 	if ( d )
 		ret = d->canStartBroadcast( live, list.at(0) );
 	else
@@ -1831,20 +1880,14 @@ void DvbPanel::playLastChannel()
 	if ( !channelsCb->isEnabled() )
 		return;
 
-	//++dvbConfig->lastChannel;
-	//if ( dvbConfig->lastChannel>channels.count() )
-	//	dvbConfig->lastChannel = 1;
-
 	for ( j=0; j<(int)channels.count(); j++ ) {
 		if ( (int)channels.at(j)->num==dvbConfig->lastChannel ) {
 			ok = true;
 			break;
 		}
 	}
-	if ( !ok ) {
-		//QTimer::singleShot( 3000, this, SLOT(playLastChannel()) );
+	if ( !ok )
 		return;
-	}
 
 	dvbZap( channels.at(j) );
 }
@@ -1858,7 +1901,7 @@ void DvbPanel::next()
 
 	QListViewItem* nextItem;
 
-	QListViewItem* playingItem = channelsCb->findItem( QString().sprintf("%04d", dvbConfig->lastChannel), 0 );
+	QListViewItem* playingItem = channelsCb->findItem( QString().sprintf("%05d", dvbConfig->lastChannel), 0 );
 
 	if ( !playingItem == 0 ) // yes, it's in the current category
 	{
@@ -1884,7 +1927,7 @@ void DvbPanel::previous()
 
 	QListViewItem* prevItem;
 
-	QListViewItem* playingItem = channelsCb->findItem( QString().sprintf("%04d", dvbConfig->lastChannel), 0 );
+	QListViewItem* playingItem = channelsCb->findItem( QString().sprintf("%05d", dvbConfig->lastChannel), 0 );
 
 	if ( !playingItem == 0 ) // yes, it's in the current category
 	{
@@ -1905,20 +1948,14 @@ void DvbPanel::previous()
 void DvbPanel::dvbZap( ChannelDesc *chan )
 {
 	int i;
-	DvbStream *d=0;
+	DvbStream *d;
 
 	if ( currentFifo.isEmpty() || isTuning )
 		return;
 
 	isTuning = true;
-	audioComb->setEnabled( false );
 	emit setTimeShiftFilename( "" );
-	for ( i=0; i<(int)dvb.count(); i++ ) {
-		if ( dvb.at(i)->getCurrentTransponder()==chan->tp ) {
-			d = dvb.at(i);
-			break;
-		}
-	}
+	d = getWorkingDvb( MODE_LIVE, chan );
 	for ( i=0; i<(int)dvb.count(); i++ ) {
 		if ( dvb.at(i)->hasLive() ) {
 			dvb.at(i)->preStopLive();
@@ -1927,7 +1964,7 @@ void DvbPanel::dvbZap( ChannelDesc *chan )
 			break;
 		}
 	}
-	finalZap( d, chan, true );
+	finalZap( d, chan );
 }
 
 
@@ -1948,45 +1985,24 @@ void DvbPanel::setTuning()
 
 
 
-void DvbPanel::finalZap( DvbStream *d, ChannelDesc *chan, bool setAudio, int napid )
+void DvbPanel::finalZap( DvbStream *d, ChannelDesc *chan )
 {
 	QString s, t;
-	int apidn=napid;
 	int i;
-	DvbStream *d1=d, *d2=0;
 
 	emit setCurrentPlugin( this );
 
 	fprintf( stderr, "Tuning to: %s / autocount: %lu\n", chan->name.latin1(), autocount );
 	QTime tm;
 	tm.start();
-	if ( !d1 ) {
-		for ( i=0; i<(int)dvb.count(); i++ ) {
-			if ( !dvb.at(i)->canSource( chan ) )
-				continue;
-			if ( dvb.at(i)->isTuned() ) {
-				if ( dvb.at(i)->getCurrentTransponder()==chan->tp ) {
-					d1 = dvb.at(i);
-					break;
-				}
-				else d2 = dvb.at(i);
-			}
-			else {
-				d1 = dvb.at(i);
-				break;
-			}
-		}
-		if ( !d1 && d2 )
-			d1 = d2;
-	}
 
-	if ( !d1 ) {
+	if ( !d ) {
 		emit dvbStop();
 		isTuning = false;
 		return;
 	}
 
-	int ret = d1->goLive( chan, currentFifo, apidn );
+	int ret = d->goLive( chan, currentFifo, dvbConfig->ringBufSize );
 
 	switch ( ret ) {
 		case DvbStream::ErrIsRecording :
@@ -2006,21 +2022,6 @@ void DvbPanel::finalZap( DvbStream *d, ChannelDesc *chan, bool setAudio, int nap
 			break;
 	}
 
-	if ( setAudio ) {
-		audioComb->clear();
-		for ( i=0; i<chan->napid; i++ ) {
-			s = t.setNum( chan->apid[i].pid );
-			if ( !chan->apid[i].lang.isEmpty() )
-				s = s+"("+chan->apid[i].lang+")";
-			if ( chan->apid[i].ac3 )
-				s = s+"(ac3)";
-			audioComb->insertItem( s );
-		}
-		audioComb->setCurrentItem( apidn );
-		if ( chan->napid>1 && ret==0 )
-			audioComb->setEnabled( true );
-	}
-
 	fprintf( stderr, "Tuning delay: %d ms\n", tm.elapsed() );
 	osdMode = 0;
 	if ( ret<1 ) {
@@ -2039,8 +2040,14 @@ void DvbPanel::finalZap( DvbStream *d, ChannelDesc *chan, bool setAudio, int nap
 		if ( channelsCb->visibleItems<5 ) {
 			resetSearch();
 		}
+
+		if ( d->liveIsRecording() )
+			recordBtn->setOn( true );
+		else
+			recordBtn->setOn( false );
 	}
 	else {
+		recordBtn->setOn( false );
 		isTuning = false;
 		emit dvbStop();
 	}
@@ -2102,13 +2109,12 @@ void DvbPanel::stopLive()
 			break;
 		}
 	}
+	recordBtn->setOn( false );
 	emit setTimeShiftFilename( "" );
-	audioComb->clear();
-	audioComb->setEnabled( false );
 }
 
 
-//   TV | vpid(stream_type) | apid1(lang)(ac3),apid2, | ttpid | sid | tsid | type(S,C,T)source | freq | sr | pol | fecH | inv | mod | fecL | bw | trans | guard | hier | number / subpid1(type)(pageid)(ancid)(lang),subpid2..., | category |
+//   TV | vpid(stream_type) | apid1(lang)(ac3),apid2, | ttpid | sid | tsid | type(S,C,T)source | freq | sr | pol | fecH | inv | mod | fecL | bw | trans | guard | hier | number / subpid1(type)(pageid)(ancid)(lang),subpid2..., | category | nid |
 bool DvbPanel::getChannelList()
 {
 	bool ret=false;
@@ -2245,6 +2251,8 @@ bool DvbPanel::getChannelList()
 				case 67 : chan->tp.coderateH = FEC_6_7; break;
 				case 78 : chan->tp.coderateH = FEC_7_8; break;
 				case 89 : chan->tp.coderateH = FEC_8_9; break;
+				case 35 : chan->tp.coderateH = FEC_3_5; break;
+				case 910 : chan->tp.coderateH = FEC_9_10; break;
 				case -1 : chan->tp.coderateH = FEC_AUTO;
 			}
 			s = s.right( s.length()-pos-1 );
@@ -2265,6 +2273,9 @@ bool DvbPanel::getChannelList()
 				case 256 : chan->tp.modulation = QAM_256; break;
 				case 108 : chan->tp.modulation = VSB_8; break;
 				case 116 : chan->tp.modulation = VSB_16; break;
+				case 1000 : chan->tp.modulation = PSK_8; break;
+				case 1001 : chan->tp.modulation = APSK_16; break;
+				case 1003 : chan->tp.modulation = DQPSK; break;
 				case -1 : chan->tp.modulation = QAM_AUTO;
 			}
 			s = s.right( s.length()-pos-1 );
@@ -2279,6 +2290,8 @@ bool DvbPanel::getChannelList()
 				case 67 : chan->tp.coderateL = FEC_6_7; break;
 				case 78 : chan->tp.coderateL = FEC_7_8; break;
 				case 89 : chan->tp.coderateL = FEC_8_9; break;
+				case 35 : chan->tp.coderateH = FEC_3_5; break;
+				case 910 : chan->tp.coderateH = FEC_9_10; break;
 				case -1 : chan->tp.coderateL = FEC_AUTO;
 			}
 			s = s.right( s.length()-pos-1 );
@@ -2343,6 +2356,20 @@ bool DvbPanel::getChannelList()
 			}
 			pos = s.find("|");
 			chan->category = s.left( pos );
+			s = s.right( s.length()-pos-1 );
+			pos = s.find("|");
+			chan->tp.nid = s.left(pos).toUShort();
+			s = s.right( s.length()-pos-1 );
+			pos = s.find("|");
+			switch ( s.left(pos).toInt() ) {
+				case 20 : chan->tp.rolloff = ROLLOFF_20; break;
+				case 25 : chan->tp.rolloff = ROLLOFF_25; break;
+				case 35 : chan->tp.rolloff = ROLLOFF_35; break;
+				case -1 : chan->tp.rolloff = ROLLOFF_AUTO;
+			}
+			s = s.right( s.length()-pos-1 );
+			pos = s.find("|");
+			chan->tp.S2 = s.left(pos).toInt();
 
 			if ( chan->tp.source.isEmpty() ) {
 				delete chan;
@@ -2350,7 +2377,7 @@ bool DvbPanel::getChannelList()
 			}
 
 			chan->pix.load( dvbConfig->dvbConfigIconsDir+chan->name );
-			it = new KListViewItem( channelsCb, QString().sprintf("%04d", chan->num), chan->name, chan->tp.source );
+			it = new KListViewItem( channelsCb, QString().sprintf("%05d", chan->num), chan->name, chan->tp.source );
 			it->setDragEnabled( true );
 			if ( !chan->pix.isNull() )
 				it->setPixmap( 1, chan->pix );
@@ -2454,6 +2481,8 @@ bool DvbPanel::saveChannelList()
 				case FEC_6_7 : tt<< "67|"; break;
 				case FEC_7_8 : tt<< "78|"; break;
 				case FEC_8_9 : tt<< "89|"; break;
+				case FEC_3_5 : tt<< "35|"; break;
+				case FEC_9_10 : tt<< "910|"; break;
 				case FEC_AUTO : tt<< "-1|";
 			}
 			switch ( chan->tp.inversion ) {
@@ -2470,6 +2499,9 @@ bool DvbPanel::saveChannelList()
 				case QAM_256 : tt<< "256|"; break;
 				case VSB_8 : tt<< "108|"; break;
 				case VSB_16 : tt<< "116|"; break;
+				case PSK_8 : tt<< "1000|"; break;
+				case APSK_16 : tt<< "1001|"; break;
+				case DQPSK : tt<< "1003|"; break;
 				case QAM_AUTO : tt<< "-1|";
 			}
 			switch ( chan->tp.coderateL ) {
@@ -2482,6 +2514,8 @@ bool DvbPanel::saveChannelList()
 				case FEC_6_7 : tt<< "67|"; break;
 				case FEC_7_8 : tt<< "78|"; break;
 				case FEC_8_9 : tt<< "89|"; break;
+				case FEC_3_5 : tt<< "35|"; break;
+				case FEC_9_10 : tt<< "910|"; break;
 				case FEC_AUTO : tt<< "-1|";
 			}
 			switch ( chan->tp.bandwidth ) {
@@ -2519,7 +2553,16 @@ bool DvbPanel::saveChannelList()
 				tt<< ",";
 			}
 			tt<< "|";
-			tt<< chan->category+"|\n";
+			tt<< chan->category+"|";
+			tt<< s.setNum(chan->tp.nid)+"|";
+			switch ( chan->tp.rolloff ) {
+				case ROLLOFF_20 : tt<< "20|"; break;
+				case ROLLOFF_25 : tt<< "25|"; break;
+				case ROLLOFF_35 : tt<< "35|"; break;
+				case ROLLOFF_AUTO : tt<< "-1|";
+			}
+			tt<< s.setNum(chan->tp.S2)+"|";
+			tt<< "\n";
 		}
 		ret = true;
 		f.close();
@@ -2557,7 +2600,7 @@ bool DvbPanel::getTimerList()
 			t->duration = QTime::fromString( s.left( pos ) );
 			s = s.right( s.length()-pos-1 );
 			pos = s.find("|");
-			t->filetype = s.left( pos ).toInt();
+			//t->filetype = s.left( pos ).toInt();
 			t->mode = 0;
 			s = s.right( s.length()-pos-1 );
 			pos = s.find("|");
@@ -2592,7 +2635,7 @@ bool DvbPanel::saveTimerList()
 			tt<< t->channel+"|";
 			tt<< t->begin.toString("yyyy-MM-ddThh:mm:ss")+"|";
 			tt<< t->duration.toString()+"|";
-			tt<< s.setNum(t->filetype)+"|";
+			tt<< s.setNum(0/*t->filetype*/)+"|";
 			tt<< s.setNum(t->mode)+"|";
 			tt<< "\n";
 		}
