@@ -38,7 +38,7 @@ class KRecord : public QDialog
 
 public:
 
-	KRecord( QStringList chanList, QPtrList<RecTimer> *t, QWidget *parent, QSize size, int format );
+	KRecord( QStringList chanList, QPtrList<RecTimer> *t, QWidget *parent, QSize size );
 	~KRecord();
 
 	QStringList channelsList;
@@ -58,7 +58,6 @@ private:
 	KPushButton *newBtn, *editBtn, *deleteBtn, *okBtn;
 	KListView *list;
 	QPtrList<RecTimer> *timers;
-	int prefFormat;
 	QPixmap isRecording, yesRepeat;
 
 signals:
