@@ -48,9 +48,11 @@ class QTime;
 
 #include <qtimer.h>
 #include <qptrlist.h>
+#include <qthread.h>
 #include <X11/Xlib.h>
 #include <xine.h>
 
+
 #ifndef USE_QT_ONLY
 #include "postfilter.h"
 #else
@@ -70,7 +72,7 @@ typedef int PostFilter; /* dummy type */
 #define OSD_MESSAGE_LOW_PRIORITY    1
 #define OSD_MESSAGE_NORMAL_PRIORITY 2
 
-class KXineWidget : public QWidget
+class KXineWidget : public QWidget, public QThread
 {
 
 	Q_OBJECT
@@ -228,6 +230,7 @@ public slots:
 	void slotSetFileSubtitles(QString);
 	void slotStartSeeking();
 	void slotSeekToPosition(int pos);
+	void slotSeekToPositionBlocking(int pos);
 	void slotSeekToTime(const QTime&);
 	void slotStopSeeking();
 	void slotEnableVideoFilters(bool enable);
@@ -310,6 +313,7 @@ protected:
 	void mouseDoubleClickEvent(QMouseEvent*);
 	void wheelEvent(QWheelEvent*);
 	void initOSD();
+	virtual void run();
 
 private:
 #ifndef HAVE_XCB
@@ -332,6 +336,8 @@ private:
 	static void showOSDMessagesChangedCallback(void* p, xine_cfg_entry_t* entry);
 	static void sizeForOSDMessagesChangedCallback(void* p, xine_cfg_entry_t* entry);
 	static void fontForOSDMessagesChangedCallback(void* p, xine_cfg_entry_t* entry);
+	static void monitorXResChangedCallback(void* p, xine_cfg_entry_t* entry);
+	static void monitorYResChangedCallback(void* p, xine_cfg_entry_t* entry);
 
 	static void xineEventListener(void* p, const xine_event_t*);
 
@@ -360,6 +366,9 @@ public:
 protected:
 	QStringList m_queue;
 
+	int seekThreadPos;
+	int seekThreadTime;
+	bool seekThreadPause;
 	bool m_startXineManual;
 	bool m_xineReady;
 	QString m_logoFile;
@@ -402,6 +411,9 @@ protected:
 	char** m_osdSizeOptions;
 	char* m_osdSizeInfo;
 	char* m_osdFontInfo;
+	double monitorXRes, monitorYRes;
+	char *xResInfo;
+	char *yResInfo;
 
 #ifndef USE_QT_ONLY
 	/*postfilter*/
